@@ -653,9 +653,9 @@ class Agent(Insight, Channel, Catalog, Dataset):
         return r
 
 
-    def add_user_to_channel(self, channel_slug, user_email, role="FOLLOWER"):
+    def __get_user_by_email(self, user_email):
 
-        url = self._base_url_askdata +"/smartbot/share/principals?q="+ user_email
+        url = self._base_url_askdata + "/smartbot/share/principals?q=" + user_email
         s = requests.Session()
         s.keep_alive = False
         retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
@@ -664,17 +664,16 @@ class Agent(Insight, Channel, Catalog, Dataset):
         r.raise_for_status()
         if (r != None and r.json() != []):
             user_id = r.json()[0]["id"]
+            return user_id
         else:
             print("Please insert valid email")
             return None
 
-        channel_id = self.get_channel(channel_slug)
-        self.__add_user_to_ch(channel_id, user_id, role)
-
     '''
-    role: FOLLOWER/ADMIN
+    role: VIEW/EDIT
     '''
-    def __add_user_to_ch(self, channel_id, user_id, role="FOLLOWER"):
+    def add_user_to_workspace(self, user_email, role="VIEW"):
+        user_id = self.__get_user_by_email(user_email)
 
         data = {
             "principals": [
@@ -686,6 +685,34 @@ class Agent(Insight, Channel, Catalog, Dataset):
             "permission": role
         }
 
+        url = self._base_url+"/smartbot/agents/{}/share/entries".format(self._agentId)
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+
+        r = s.post(url=url, headers=self._headers, json=data)
+        r.raise_for_status()
+
+
+    '''
+    role: FOLLOWER/ADMIN
+    '''
+    def add_user_to_channel(self, channel_slug, user_email, role="FOLLOWER"):
+
+        user_id = self.__get_user_by_email(user_email)
+
+        channel_id = self.get_channel(channel_slug)
+        self.__add_user_to_ch(channel_id, user_id, role)
+
+    def __add_user_to_ch(self, channel_id, user_id, role="FOLLOWER"):
+
+        data = {
+            "userId":user_id,
+            "role":role,
+            "mute":"none"
+        }
+
         s = requests.Session()
         s.keep_alive = False
         retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
