@@ -297,6 +297,29 @@ class Query:
 
     def _where_spell_out(self, where):
         conditions = []
+
+        consistent_datatypes = {"numeric":
+                                    ["bit", "smallmoney", "money", "int",
+                                     "decimal", "float", "float64", "double",
+                                     "bigdecimal", "int32", "int64", "bigint",
+                                     "numeric", "bignumeric", "smallint", "integer",
+                                     "tinyint", "byteint", "int8", "long",
+                                     "float8", "hugeint", "int4", "signed",
+                                     "real", "float4", "int2", "short",
+                                     "int1", "ubigint", "uinteger", "usmallint",
+                                     "utinyint", "bool", "boolean", "mediumint",
+                                     "double precision", "dec", "smallserial", "serial",
+                                     "bigserial", "number"],
+                                "date":
+                                    ["date", "datetimeoffset", "datetime2", "smalldatetime",
+                                     "timestamp", "datetime", "time", "year",
+                                     "interval", "timestamp_ltz", "timestamp_ntz", "timestamp_tz"],
+                                "string":
+                                    ["string", "varchar", "varchar2", "char",
+                                     "bpchar", "text", "nchar", "nvarchar",
+                                     "ntext", "tinytext", "mediumtext", "longtext",
+                                     "character", "character varying", "binary", "varbinary"]}
+
         for where_condition in where:
             if isinstance(where_condition, Condition):
                 vars = []
@@ -324,12 +347,54 @@ class Query:
                         operator = "<"
                 else:
                     operator = None
-                if where_condition.field.internalDataType is not None:
-                    formatted_cond = where_condition.field.column + " (" + where_condition.field.internalDataType + ") " + operator + " " + formatted_value
-                elif where_condition.field.sourceDataType is not None:
-                    formatted_cond = where_condition.field.column + " (" + where_condition.field.sourceDataType + ") " + operator + " " + formatted_value
+
+                consistent_vec = consistent_datatypes[where_condition.field.internalDataType.lower()]
+                if (
+                        where_condition.field.internalDataType.lower() == "numeric" or where_condition.field.internalDataType.lower() == "string") \
+                        and where_condition.field.sourceDataType.lower() not in consistent_vec:
+                    # if internal and original data types are not consistent each other use this pattern
+                    # aggregation column (original data type, to internal data type ) AS Alias
+                    if where_condition.field.aggregation is not None and isinstance(
+                            where_condition.field.aggregation, str):
+                        formatted_cond = where_condition.field.aggregation + " " + where_condition.field.column + " (" + where_condition.field.sourceDataType + ", to " + where_condition.field.internalDataType + ") " + operator + " " + formatted_value
+                    elif where_condition.field.aggregation is not None and isinstance(
+                            where_condition.field.aggregation, SQLFunction):
+                        formatted_cond = where_condition.field.aggregation.name + " " + where_condition.field.column + " (" + where_condition.field.sourceDataType + ", to " + where_condition.field.internalDataType + ") " + operator + " " + formatted_value
+                    else:
+                        formatted_cond = where_condition.field.column + " (" + where_condition.field.sourceDataType + ", to " + where_condition.field.internalDataType + ") " + operator + " " + formatted_value
+                elif where_condition.field.internalDataType.lower() == "date" and where_condition.field.sourceDataType.lower() not in consistent_vec:
+                    # if internal and original data types are not consistent each other use this pattern
+                    # aggregation column (original data type, to internal data type, date_format YYYY-MM-DD ) AS Alias
+                    if where_condition.field.sourceFormat is not None:
+                        if where_condition.field.aggregation is not None and isinstance(
+                                where_condition.field.aggregation, str):
+                            formatted_cond = where_condition.field.aggregation + " " + where_condition.field.column + " (" + where_condition.field.sourceDataType + ", to " + where_condition.field.internalDataType + ", format " + where_condition.field.sourceFormat + ") " + operator + " " + formatted_value
+                        elif where_condition.field.aggregation is not None and isinstance(
+                                where_condition.field.aggregation, SQLFunction):
+                            formatted_cond = where_condition.field.aggregation.name + " " + where_condition.field.column + " (" + where_condition.field.sourceDataType + ", to " + where_condition.field.internalDataType + ", format " + where_condition.field.sourceFormat + ") " + operator + " " + formatted_value
+                        else:
+                            formatted_cond = where_condition.field.column + " (" + where_condition.field.sourceDataType + ", to " + where_condition.field.internalDataType + ", format " + where_condition.field.sourceFormat + ") " + operator + " " + formatted_value
+                    else:
+                        if where_condition.field.aggregation is not None and isinstance(
+                                where_condition.field.aggregation, str):
+                            formatted_cond = where_condition.field.aggregation + " " + where_condition.field.column + " (" + where_condition.field.sourceDataType + ", to " + where_condition.field.internalDataType + ", format YYYY-MM-DD" + ") " + operator + " " + formatted_value
+                        elif where_condition.field.aggregation is not None and isinstance(
+                                where_condition.field.aggregation, SQLFunction):
+                            formatted_cond = where_condition.field.aggregation.name + " " + where_condition.field.column + " (" + where_condition.field.sourceDataType + ", to " + where_condition.field.internalDataType + ", format YYYY-MM-DD" + ") " + operator + " " + formatted_value
+                        else:
+                            formatted_cond = where_condition.field.column + " (" + where_condition.field.sourceDataType + ", to " + where_condition.field.internalDataType + ", format YYYY-MM-DD" + ") " + operator + " " + formatted_value
                 else:
-                    formatted_cond = where_condition.field.column + " " + operator + " " + formatted_value
+                    # else
+                    if where_condition.field.aggregation is not None and isinstance(
+                            where_condition.field.aggregation, str):
+                        formatted_cond = where_condition.field.aggregation + " " + where_condition.field.column + " (" + where_condition.field.sourceDataType + ") " + operator + " " + formatted_value
+                    elif where_condition.field.aggregation is not None and isinstance(
+                            where_condition.field.aggregation,
+                            SQLFunction):
+                        formatted_cond = where_condition.field.aggregation.name + " " + where_condition.field.column + " (" + where_condition.field.sourceDataType + ") " + operator + " " + formatted_value
+                    else:
+                        formatted_cond = where_condition.field.column + " (" + where_condition.field.sourceDataType + ") " + operator + " " + formatted_value
+
                 conditions.append(formatted_cond)
             elif isinstance(where_condition, CompositeCondition):
                 other_conds = self._where_spell_out(where_condition.conditions)
@@ -342,9 +407,27 @@ class Query:
     def spell_out(self):
         spell_str = '# {"fields":['
 
-        consistent_datatypes = {"numeric": ["int", "float", "double", "int32", "int64"],
-                                "date": ["date", "timestamp", "datetime"],
-                                "string": ["string", "varchar", "varchar2"]}
+        consistent_datatypes = {"numeric":
+                                    ["bit", "smallmoney", "money", "int",
+                                     "decimal", "float", "float64", "double",
+                                     "bigdecimal", "int32", "int64", "bigint",
+                                     "numeric", "bignumeric", "smallint", "integer",
+                                     "tinyint", "byteint", "int8", "long",
+                                     "float8", "hugeint", "int4", "signed",
+                                     "real", "float4", "int2", "short",
+                                     "int1", "ubigint", "uinteger", "usmallint",
+                                     "utinyint", "bool", "boolean", "mediumint",
+                                     "double precision", "dec", "smallserial", "serial",
+                                     "bigserial", "number"],
+                                "date":
+                                    ["date", "datetimeoffset", "datetime2", "smalldatetime",
+                                     "timestamp", "datetime", "time", "year",
+                                     "interval", "timestamp_ltz", "timestamp_ntz", "timestamp_tz"],
+                                "string":
+                                    ["string", "varchar", "varchar2", "char",
+                                     "bpchar", "text", "nchar", "nvarchar",
+                                     "ntext", "tinytext", "mediumtext", "longtext",
+                                     "character", "character varying", "binary", "varbinary"]}
 
         fields = []
         for field in self.fields:
@@ -367,7 +450,8 @@ class Query:
             else:
                 # otherwise, check if internal and original data types are not consistent each other
                 consistent_vec = consistent_datatypes[field.internalDataType.lower()]
-                if (field.internalDataType.lower() == "numeric" or field.internalDataType.lower() == "string") and field.sourceDataType.lower() not in consistent_vec:
+                if (
+                        field.internalDataType.lower() == "numeric" or field.internalDataType.lower() == "string") and field.sourceDataType.lower() not in consistent_vec:
                     # if internal and original data types are not consistent each other use this pattern
                     # aggregation column (original data type, to internal data type ) AS Alias
                     if field.alias is None:
